

ENStats.version = tostring(GetAddOnMetadata("EN_Stats", "Version"))
local L = LibStub("AceLocale-3.0"):GetLocale("ENStats", true)
local media = LibStub("LibSharedMedia-3.0")
local fail = LibStub("LibFail-1.0")

local players={}
local fail_events = fail:GetSupportedEvents()

-- fix possible errors from unavailable localizations (new LibFail events)
for _,v in ipairs(fail_events) do
	if not L[v] then L[v]=v end
end

local mode = 0

local defaults = {
	global = {
		colorbg = {
			["r"] = 0,
			["g"] = 0,
			["b"] = 0,
			["a"] = 1,		 
		},	
		color = {
			["r"] = 0.5,
			["g"] = 0.5,
			["b"] = 0.7,
			["a"] = 1,		 
		},	
		texcolor = {
			["r"] = 1,
			["g"] = 1,
			["b"] = 1,
			["a"] = 1,		 
		},
		MiniMap={},
		output = "SAY",
		height = 16,
		width = 230,
		fontsize = 12,
		font = 1,	
		tex = 1,
		anno = true,
		channel = "<enter channel number>",
		annochannel = "<enter channel number>",
		annooutput = "SAY",
		monthly = {},
		strata="HIGH",
		vis = true,
	},
}

local function tcount(tab)
	if(tab == nil) then 
		return 0 
	end
	local n=0
	for _ in pairs(tab) do
		n=n+1
	end
	return n
end

function ENStats:Fail(event, name, ...)
	if not players[name] then players[name]={flip=0,events={},tt=GetTime()-10} end
	if not players[name].events[event] then players[name].events[event] = 0 end
	if GetTime()-players[name].tt<0.7 then return end
	players[name].tt = GetTime()
    players[name].events[event] = players[name].events[event] + 1
    players[name].flip = players[name].flip  + 1  
    if dbd.anno then SendChatMessage(name..L[" has failed at "]..L[event], dbd.annooutput, string.upper(GetDefaultLanguage()), dbd.annochannel) end
    if string.find(name, "Test")==nil then 
	    if not dbd.monthly[name] then dbd.monthly[name]={flip=0,events={}} end
		if not dbd.monthly[name].events[event] then dbd.monthly[name].events[event] = 0 end
	    dbd.monthly[name].events[event] = dbd.monthly[name].events[event] + 1
	    dbd.monthly[name].flip = dbd.monthly[name].flip  + 1
    end   
    if ENStats.header:IsVisible() then self:UpdateFrame() end	     
end

local function CreateBar(i)
	ENStats.bars[i] = CreateFrame("StatusBar",nil,ENStats.container)
	ENStats.bars[i]:SetStatusBarTexture(media:Fetch("statusbar",ENStats.db.global.tex))
	ENStats.bars[i]:SetWidth(ENStats.header:GetWidth()-27)
	ENStats.bars[i]:SetHeight(dbd.height)
	ENStats.bars[i]:SetStatusBarColor(dbd.color.r, dbd.color.g, dbd.color.b, dbd.color.a)
	ENStats.bars[i]:SetMinMaxValues(0, 100)
	ENStats.bars[i]:SetValue(100)
	ENStats.bars[i]:SetPoint("TOPLEFT",8,(i*-dbd.height)+dbd.height-7)
	ENStats.bars[i]:EnableMouse(true)
	ENStats.bars[i]:SetScript('OnLeave', function() GameTooltip:Hide() end)
	ENStats.bars[i]:Hide()
	ENStats.LeftText[i] = ENStats.bars[i]:CreateFontString(nil, 'OVERLAY')		
	ENStats.LeftText[i]:SetTextColor(dbd.texcolor.r,dbd.texcolor.g,dbd.texcolor.b,dbd.texcolor.a)		
	ENStats.LeftText[i]:SetPoint("LEFT", ENStats.bars[i], 4, 0)
	ENStats.RightText[i] = ENStats.bars[i]:CreateFontString(nil, 'OVERLAY')		
	ENStats.RightText[i]:SetTextColor(dbd.texcolor.r,dbd.texcolor.g,dbd.texcolor.b,dbd.texcolor.a)		
	ENStats.RightText[i]:SetPoint("RIGHT", ENStats.bars[i], -4, 0)		
	ENStats.LeftText[i]:SetFont(media:Fetch("font",dbd.font),dbd.fontsize)
	ENStats.RightText[i]:SetFont(media:Fetch("font",dbd.font),dbd.fontsize)	
	ENStats.bars[i]:EnableMouse(true)				
end

-- this function needs cleanup!
function ENStats:UpdateFrame()
	local data
	if mode == 0 then data = players else data = dbd.monthly end	
	local temp={}	
	local count=0
	for k,v in pairs(data) do
		count=count+1		
		temp[count]={["flip"]=v.flip, ["name"]=k}
	end
	count = 0
	table.sort(temp, function(a,b) return a["flip"] > b["flip"] end)
	for i=1,#ENStats.bars do
		ENStats.bars[i]:Hide()
	end
	for _,v in ipairs(temp) do		
		count=count+1
		if not ENStats.bars[count] then CreateBar(count) end
		ENStats.bars[count]:Show()
		ENStats.LeftText[count]:SetText(count..") "..v.name)
		ENStats.RightText[count]:SetText(v.flip)
		ENStats.bars[count]:SetValue((v.flip/temp[1].flip)*100)		
		ENStats.bars[count]:SetScript('OnMouseUp', function() if IsShiftKeyDown() then ENStats:DoReport(v.name,true) else ENStats:DoReport(v.name) end end)
		ENStats.bars[count]:SetScript('OnEnter', function() 	
			GameTooltip:ClearLines()
			GameTooltip:SetOwner(ENStats.bars[count], "ANCHOR_CURSOR")
			GameTooltip:AddLine("|cff7fff7f"..v.name.."|r")
			GameTooltip:AddLine(v.flip..L[" Flip Points"])
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine("|cff7fff7f"..L["Summary of failures"].."|r:")
			for u,z in pairs(data[v.name].events) do
					GameTooltip:AddLine(z.."x "..L[u])
			end
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine(L["Click: Report to chat"])
			GameTooltip:AddLine(L["Shift + Click: Report to target"])
			GameTooltip:Show()
		end)
	end
	if count==0 then 
		count=2 
		ENStats.bars[1]:SetValue(0)
		ENStats.bars[1]:SetScript('OnEnter', nil)
		ENStats.bars[1]:SetScript('OnMouseUp', nil)
		ENStats.LeftText[1]:SetText(L["Currently no noobs detected."])
		ENStats.RightText[1]:SetText("")
		ENStats.bars[1]:Show()
	end
	ENStats.container:SetHeight((count*dbd.height)+13)
	ENStats.MainFrame:UpdateScrollChildRect()
end		

function ENStats:OnCmd(arg1)
	local arg1,arg2=string.split(" ", arg1)
	if string.lower(arg1) == "menu" then
		ENStats:OnFuBarClick("RightButton")
	elseif string.lower(arg1)=="minimap" and not IsAddOnLoaded("FuBar") then
		if type(tonumber(arg2))~="number" then
				dbd.MiniMap.hide = not dbd.MiniMap.hide
				ENStats:ToggleMM(dbd.MiniMap.hide)
		else
			if tonumber(arg2)>=0 and tonumber(arg2)<=360 then
				dbd.MiniMap.minimapPos = tonumber(arg2)
				ENStats:ToggleMM(dbd.MiniMap.hide)
			else
				BossTalk:Print(L["Value must be smaller than 360 and greater than 0!"])
			end
		end
	elseif string.lower(arg1)=="test" then	
		for i=1,25 do
			local e = math.floor(math.random() * #fail_events) + 1
    		local p = math.floor(math.random() * 10) + 1
    		self:Fail(fail_events[e], "Test"..p)
		end
		ENStats.MainFrame:Show()	
	else
		ENStats:OnFuBarClick("LeftButton")
	end
end	
									
function ENStats:OnEnable()
	ENStats:RegisterChatCommand('ENStats', 'OnCmd') 
	self:Print("v"..ENStats.version.." loaded.")	
	
	ENStats.MainFrame:SetScript("OnShow", function() dbd.vis=true ENStats:UpdateFrame() end)
	ENStats.MainFrame:SetScript("OnHide", function() dbd.vis=false ENStats:UpdateFrame() end)
	ENStats.header = CreateFrame("Frame", nil, ENStats.MainFrame)
	ENStats.header:SetWidth(ENStats.MainFrame:GetWidth())
	ENStats.header:SetHeight(26)	
	ENStats.header:EnableMouse(true)
	ENStats.header:SetHitRectInsets(1, 30, 1, 1)
	ENStats.header:RegisterForDrag("LeftButton")
	ENStats.header:SetPoint("TOPLEFT", ENStats.MainFrame, "TOPLEFT", 0, 21)
	ENStats.header:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
		tile = true, tileSize = 16, edgeSize = 16, 
		insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	ENStats.header:SetBackdropColor(dbd.colorbg.r, dbd.colorbg.g, dbd.colorbg.b, dbd.colorbg.a)		
	ENStats.header:SetScript("OnDragStart", function()		
		if(arg1 == "LeftButton") then			
			ENStats.MainFrame:StartMoving()
			ENStats.MainFrame:SetUserPlaced(true)	
		end
	end)		
	ENStats.header:SetScript("OnDragStop", function()
		ENStats.MainFrame:StopMovingOrSizing()
		ENStats.MainFrame:SetUserPlaced(true)	
	end)
	
	local closebutton = CreateFrame("Button", nil, ENStats.MainFrame, "UIPanelCloseButton")
	closebutton:SetPoint("TOPRIGHT", self.header, "TOPRIGHT", 3, 3)
	closebutton:SetScale(1)
	closebutton:SetScript('OnClick', function() ENStats.MainFrame:Hide() end)
	
	local Titel = ENStats.header:CreateFontString(nil, 'OVERLAY')		
	Titel:SetTextColor(0.95,0.95,0,1)		
	Titel:SetPoint("LEFT", ENStats.header, 10, 0)
	Titel:SetFont("Fonts\\FRIZQT__.TTF",15)
	Titel:SetText("Endeavour's Noob Stats")
		
	ENStats.bars={}
	ENStats.LeftText={}
	ENStats.RightText={}
	CreateBar(1)
	
	ENStats.footer = CreateFrame("Frame", nil, ENStats.MainFrame)
	ENStats.footer:SetWidth(ENStats.MainFrame:GetWidth())
	ENStats.footer:SetHeight(24)	
	ENStats.footer:SetPoint("TOP", ENStats.MainFrame, "BOTTOM",0,5)
	ENStats.footer:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
		tile = true, tileSize = 16, edgeSize = 16, 
		insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	ENStats.footer:SetBackdropColor(dbd.colorbg.r, dbd.colorbg.g, dbd.colorbg.b, dbd.colorbg.a)

	local prev = CreateFrame("Button", nil, ENStats.MainFrame)	
	prev:SetNormalTexture("Interface\\BUTTONS\\UI-GuildButton-PublicNote-Up")
	prev:SetHighlightTexture("Interface\\BUTTONS\\UI-GuildButton-PublicNote-Disabled")	
	prev:SetWidth(16)
	prev:SetHeight(16)
	prev:SetPoint("LEFT", self.footer, "LEFT", 36, 0)
	prev:SetScript('OnClick', function() 
		if mode == 1 then
			mode=0
			ENStats.btext:SetText(L["Current Data"])
		elseif mode == 0 then
			mode=1			
			ENStats.btext:SetText(L["Overall Data"])
		end
		self:UpdateFrame()
	end)	
	prev:SetScript('OnEnter', function() 	
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(prev, "ANCHOR_BOTTOMLEFT")
		GameTooltip:AddLine(L["Switch Data"])
		GameTooltip:SetScale(0.8)
		GameTooltip:Show()
	end)
	prev:SetScript('OnLeave', function() GameTooltip:Hide() GameTooltip:SetScale(1) end)
	
	local reset = CreateFrame("Button", nil, ENStats.MainFrame)	
	reset:SetNormalTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up")
	reset:SetHighlightTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Highlight", "ADD")	
	reset:SetWidth(16)
	reset:SetHeight(16)
	reset:SetPoint("LEFT", self.footer, "LEFT", 4, 0.5)
	reset:SetScript('OnClick', function()
		if mode == 0 then 
			players={}
			self:UpdateFrame()
		else
			StaticPopupDialogs["ENStats_Confirmation"] = { text = L["Do you really want to delete all data?"], button1 = "Yes", button2 = "No", OnAccept = function() dbd.monthly={}self:UpdateFrame() end, timeout = 0, whileDead = 1, hideOnEscape = 1 }
			StaticPopup_Show("ENStats_Confirmation")
		end  		 
	end)
	reset:SetScript('OnEnter', function() 	
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(reset, "ANCHOR_BOTTOMLEFT")
		GameTooltip:AddLine("Reset")
		GameTooltip:SetScale(0.8)
		GameTooltip:Show()
	end)
	reset:SetScript('OnLeave', function() GameTooltip:Hide() GameTooltip:SetScale(1) end)
	
	local report = CreateFrame("Button", nil, ENStats.MainFrame)
	report:SetNormalTexture("Interface\\BUTTONS\\UI-GuildButton-MOTD-Up")
	report:SetHighlightTexture("Interface\\BUTTONS\\UI-GuildButton-MOTD-Disabled")	
	report:SetWidth(16); report:SetHeight(16); 
	report:SetPoint("LEFT", self.footer, "LEFT", 20, 0.5)
	report:SetScript('OnClick', function() if IsShiftKeyDown() then ENStats:DoReport(nil, true) else ENStats:DoReport() end end)
	report:SetScript('OnEnter', function() 	
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(report, "ANCHOR_BOTTOMLEFT")
		GameTooltip:AddLine(L["Click: Report to chat"])
		GameTooltip:AddLine(L["Shift + Click: Report to target"])
		GameTooltip:SetScale(0.8)
		GameTooltip:Show()
	end)
	report:SetScript('OnLeave', function() GameTooltip:Hide() GameTooltip:SetScale(1) end)
	
	
	local sizer = CreateFrame("Button", nil, ENStats.footer)
	sizer:SetNormalTexture("Interface\\CURSOR\\Item")
	sizer:SetHighlightTexture("Interface\\CURSOR\\Item")	
	sizer:GetNormalTexture():SetTexCoord(1,0,1,0)
	sizer:GetHighlightTexture():SetTexCoord(1,0,1,0)
	sizer:SetHitRectInsets(10, 0, 10, 0) 
    sizer:GetNormalTexture():SetVertexColor(1, 1, 1, 0.3)    
	sizer:SetWidth(32)
	sizer:SetHeight(32) 
	sizer:SetPoint("RIGHT", self.footer, "RIGHT", -4, 8.5)
	sizer:SetScript('OnMouseDown', function()
		sizer:SetScript('OnUpdate', function() 
			ENStats.header:SetWidth(ENStats.MainFrame:GetWidth()) 
			ENStats.footer:SetWidth(ENStats.MainFrame:GetWidth())
			ENStats.container:SetWidth(ENStats.MainFrame:GetWidth())
			for i=1,#ENStats.bars do
				ENStats.bars[i]:SetWidth(ENStats.MainFrame:GetWidth()-27)
			end 
		end) 
		ENStats.MainFrame:StartSizing() 
	end)
	sizer:SetScript('OnMouseUp', function() ENStats.MainFrame:StopMovingOrSizing() sizer:SetScript('OnUpdate', nil) dbd.width = ENStats.MainFrame:GetWidth() ENStats.MainFrame:SetUserPlaced(true) end)
	
	ENStats.btext = ENStats.footer:CreateFontString(nil, 'OVERLAY')		
	ENStats.btext:SetTextColor(0.95,0.95,0,1)		
	ENStats.btext:SetPoint("LEFT", self.footer, "LEFT", 53, 1)
	ENStats.btext:SetFont("Fonts\\FRIZQT__.TTF",11)
	ENStats.btext:SetText(L["Current Data"])
	
	for _, v in pairs(fail_events) do
		fail.RegisterCallback(self, v, "Fail")        
	end
	self:UpdateFrame()
end		

function ENStats:DoReport(arg1,arg2)
	local chat = {[1]="Endeavour's Noob Stats",}	
	local data
	if mode == 0 then data = players else data = dbd.monthly end	
	if not arg1 then
		local temp = {}
		local counter=0
		for k,v in pairs(data) do
			counter=counter+1
			temp[counter]={["flip"]=v.flip, ["name"]=k}
		end
		table.sort(temp, function(a,b) return a["flip"] > b["flip"] end)
		counter=0
		for k,v in pairs(temp) do
			if k > 10 then break end
			chat[#chat+1] = k..") "..v.name..": "..v.flip.." Flips"			
		end				
	else
		chat[#chat+1] = L["Summary of failures"].." ("..arg1..")"	
		for u,z in pairs(data[arg1].events) do
			chat[#chat+1] = z.."x "..L[u]			
		end
	end
	if arg2 then 
		for _,v in ipairs(chat) do
			SendChatMessage(v, "WHISPER", string.upper(GetDefaultLanguage()), UnitName("target") or "<No Target>")
		end
	else
		for _,v in ipairs(chat) do
			SendChatMessage(v, dbd.output, string.upper(GetDefaultLanguage()), dbd.channel)
		end
	end
end

function ENStats:OnInitialize()
	ENStats.db = LibStub('AceDB-3.0'):New('ENStatsDB')
	ENStats.db:RegisterDefaults(defaults)
	dbd = ENStats.db.global
	if dbd.output == "CHANNEL" then ENStats.options.args.general.args.channel.disabled = false else ENStats.options.args.general.args.channel.disabled = true end
	if dbd.annooutput == "CHANNEL" then ENStats.options.args.general.args.annochannel.disabled = false else ENStats.options.args.general.args.annochannel.disabled = true end
	if dbd.anno then ENStats.options.args.general.args.annoreport.disabled = false end
	self:RegisterEvent("PLAYER_LOGIN")

	ENStats.MainFrame = CreateFrame("ScrollFrame", "ENStatsMainFrame", UIParent, "UIPanelScrollFrameTemplate")
	ENStats.container = CreateFrame("Frame", "ENStatsContainer", ENStats.MainFrame)
	ENStats.MainFrame:SetScrollChild(ENStats.container)	
	ENStats.MainFrame:SetVerticalScroll(dbd.height)
	ENStats.MainFrame:SetWidth(dbd.width)
	if not ENStats.MainFrame:IsUserPlaced() then ENStats.MainFrame:SetPoint("CENTER",0,20) end
	ENStats.container:SetAllPoints(ENStats.MainFrame)
	ENStats.container:SetWidth(dbd.width)
	ENStats.container:SetHeight(dbd.height-34)
	ENStats.MainFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
		tile = false, tileSize = 16, edgeSize = 16, 
		insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	ENStats.MainFrame:SetBackdropColor(dbd.colorbg.r, dbd.colorbg.g, dbd.colorbg.b, dbd.colorbg.a)					
	ENStats.MainFrame:SetHeight((25*dbd.height)+13)
	ENStats.MainFrame:SetResizable(true)
	ENStats.MainFrame:SetFrameStrata(dbd.strata)
	ENStats.MainFrame:SetClampedToScreen(true)
	ENStats.MainFrame:SetToplevel(true)
	ENStats.MainFrame:SetMovable(true)
	ENStats.MainFrame:SetMinResize(230, 29)	
	ENStatsMainFrameScrollBar:SetScale(0.7)
    ENStatsMainFrameScrollBar:SetPoint("TOPLEFT", ENStats.MainFrame, "TOPRIGHT", -24,-25)
	ENStatsMainFrameScrollBar:SetPoint("BOTTOMLEFT", ENStats.MainFrame, "BOTTOMRIGHT", -24,24)
	if dbd.vis == true then ENStats.MainFrame:Show() else ENStats.MainFrame:Hide() end 
end